import pathlib
import datetime as dt
import json
from tabulate import tabulate
from PIL import Image

from stoffel.settings import *
from stoffel.core import msft, ggl
from stoffel.core.utils.paths import path_leaf
from stoffel.core.utils.project import SOURCES, DESTINATIONS

class Source:
    def __init__(self, sheet, rng, **kwargs):
        self.sheet = sheet
        self.rng = rng
        for key, value in kwargs.items():
            self.__dict__[key] = value

    @staticmethod
    def temp_image_name(filename, sheet, rng, datetime=False):
        if datetime:
            save_to = f"{filename}_{sheet}_{rng.replace(':', '_')}_{dt.datetime.now().strftime('%Y_%m_%dT%H_%M_%S')}.jpg"
        else:
            save_to = f"{filename}_{sheet}_{rng.replace(':', '_')}.jpg"
        return save_to

class Excel(Source):
    tp = "excel"
    excel_path = None
    temp_directory = pathlib.Path(__file__).parent.parent.absolute() / "files"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        print(str(pathlib.Path(self.excel_path)))
        self.file_name, self.file_extension = str(pathlib.Path(path_leaf(self.excel_path))).split(".")

    def get(self):
        temp_image_name = self.temp_image_name(self.file_name, self.sheet, self.rng)
        wb = msft.Excel(self.excel_path)
        wb.get_image(self.sheet, self.rng, str(self.temp_directory / temp_image_name))
        if self.delete:
            wb.delete()
        return str(self.temp_directory / temp_image_name)

class Sheets(Source):
    tp = "sheets"
    sheet_id = None
    temp_directory = pathlib.Path(__file__).parent.parent.absolute() / "files"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.drive = ggl.Drive()

    def get(self):
        temp_excel_path = str(self.temp_directory / f"{self.sheet_id}.xlsx")
        self.drive.download_file(self.sheet_id, temp_excel_path)
        ex = Excel(self.sheet, self.rng, excel_path=temp_excel_path, delete=True)
        return ex.get()

class Destination:
    
    def __init__(self, w, h, x, y, resize,**kwargs):
        self.w=w
        self.h=h
        self.x=x
        self.y=y
        self.resize=resize
        for key, value in kwargs.items():
            self.__dict__[key] = value

class Slides(Destination):
    tp = "slides"
    presentation_id = None
    page_id = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.slides = ggl.Slides(self.presentation_id)
        self.drive = ggl.Drive()

    def upload(self, image_path):
        if self.resize=="checked": #Not resized, it matches the width. 
            im = Image.open(image_path)
            im = im.resize((int(self.w * 72), int(self.h * 72)))
            im.save(image_path)
        folder = self.drive.find_file(PROJECT_NAME + "Temp")
        if not folder:
            folder_id = self.drive.create_folder(PROJECT_NAME + "Temp").get("id")
        else:
            folder_id = folder[0].get("id")
        image_id = self.drive.create_image(image_path, parent=folder_id).get("id")
        self.drive.make_shareable(image_id)
        self.slides.add_image(image_id, self.page_id, self.w, self.h, self.x, self.y)
        self.drive.delete_file(image_id)
        
class Powerpoint(Destination):
    tp = "powerpoint"
    presentation_path = None
    slide_index = None
    slide_id = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        
    def upload(self, image_path):
        pt = msft.Powerpoint(self.presentation_path)
        pt.open()
        print(self.slide_index)
        if self.slide_index:
            self.slide_id = pt.get_slide_id(self.slide_index)
        print(self.slide_id)
        pt.close()
        pt.add_image(image_path, self.slide_id, self.w, self.h, self.x, self.y)

class Connection:

    def __init__(self, source, destination):
        self.source = source
        self.destination = destination

    def run(self):
        image_path = self.source.get()
        self.destination.upload(image_path)

class Project:
    sources = SOURCES
    sources["objects"] =  {
        "excel" : Excel, 
        "sheets" : Sheets,
    }
    destinations = DESTINATIONS
    destinations["objects"] = {
        "slides" : Slides, 
        "powerpoint" : Powerpoint
    }

    def __init__(self, name):
        self.name=name
        self.connections = {}
        with open(CONNECTIONS_PATH, "r") as f:
            data = json.load(f)
        
        if name in data:
            self.project = data[name]
            for id, connection in self.project.items():
                source_data = connection.get("source")
                source = self.sources["objects"][source_data.get("tp")](**source_data)
                destination_data = connection.get("destination")
                destination = self.destinations["objects"][destination_data.get("tp")](**destination_data)
                self.connections[id] = Connection(
                    source=source, 
                    destination=destination, 
                )
        else:
            self.project = {}
        
    def add_connection(self, connection_object):
        self.project[len(self.connections.keys())] = {
            "source" : {**{"tp" : connection_object.source.tp}, **{k : v for k, v in connection_object.source.__dict__.items() if k in self.sources["fields"][connection_object.source.tp]}}, 
            "destination" : {**{"tp" : connection_object.destination.tp}, **{k : v for k, v in connection_object.destination.__dict__.items() if k in self.destinations["fields"][connection_object.destination.tp]}}
        }
        self.connections[len(self.connections.keys())] = connection_object

    def show_connections(self):
        table = []
        for id, connection in self.project.items():
            source = connection.get("source")
            source_type = source.get("tp")
            source_file = path_leaf(source.get("excel_path")) if source_type == "excel" else source.get("sheet_id")
            source_sheet = source.get("sheet")
            source_range = source.get("rng")
            dest = connection.get("destination")
            dest_type = dest.get("tp")
            dest_file = path_leaf(dest.get("presentation_path")) if dest_type == "powerpoint" else dest.get("presentation_id")
            table.append([id, source_type, source_file, source_sheet, source_range, dest_type, dest_file])
        print(tabulate(table, ["id", "source_type", "source_file", "source_sheet", "source_range", "dest_type", "dest_file"], tablefmt="github"))
            
    def update(self):
        with open(CONNECTIONS_PATH, 'r+') as f:
            data = json.load(f)
            data[self.name] = self.project
            f.seek(0)
            json.dump(data, f, indent=4)
            f.truncate()  

    def remove_connection(self, id):
        self.project.pop(str(id))

    def run(self):
        for id, connection in self.connections.items():
            connection.run()

if __name__=="__main__":
    project = Project("test")

    '''
    #Connection 1
    directory = pathlib.Path(__file__).parent.parent.absolute() / "test"
    source = Excel("Sheet1", "B2:D6", excel_path=str(directory / "test.xlsx"))
    destination = Slides(3.79, 1.83, 3.1, 1.38, 
        presentation_id="1FJkzWBdb2AHL2-PKgu3BWFYtW9k1TK7OewTzvXX-BLI", 
        page_id="g7d4b0c9081_0_74", 
    )
    connection = Connection(source, destination)
    
    project.add_connection(connection)
    

    #Connection 2
    source = Sheets("Sheet2", "B3:E7", sheet_id="1Q1cm8MxDq0bzmok9PYzQc2ZjGSzNYRvjg5Yqggi_0f4")
    destination = Powerpoint(-1, -1, 3.1, 1.38, 
        presentation_path=str(directory / "Presentation1.pptx"), 
        slide_index=2
    )
    connection = Connection(source, destination)
    '''
    project.show_connections()
    project.run()



RANGE_TO_IMAGE = """
Sub RangetoImage(sheet_string As String, rng_string As String, save_path as String)
    Dim tmpChart As Chart, n As Long, shCount As Long, sht As Worksheet, sh As Shape
    Dim fileSaveName As Variant, pic As Variant
    Dim rng As Range
    
    'Create temporary chart as canvas
    Set sht = Worksheets(sheet_string)
    sht.Activate
    Set rng = sht.Range(rng_string)
    rng.Select
    Selection.Copy
    sht.Pictures.Paste.Select
    Set sh = sht.Shapes(sht.Shapes.Count)
    Set tmpChart = Charts.Add
    tmpChart.ChartArea.Clear
    tmpChart.Name = "PicChart" & (Rnd() * 10000)
    Set tmpChart = tmpChart.Location(Where:=xlLocationAsObject, Name:=sht.Name)
    tmpChart.ChartArea.Width = sh.Width
    tmpChart.ChartArea.Height = sh.Height
    tmpChart.Parent.Border.LineStyle = 0
    
    'Paste range as image to chart
    sh.Copy
    tmpChart.ChartArea.Select
    tmpChart.Paste
    
    'Save chart image to file
    tmpChart.Export Filename:=save_path, FilterName:="jpg"
    
    'Clean up
    sht.Cells(1, 1).Activate
    sht.ChartObjects(sht.ChartObjects.Count).Delete
    sh.Delete
    
End Sub
"""


from flask import Flask, render_template, g, request, redirect, make_response, jsonify
import json 
import os
import urllib
from pathlib import Path
import requests as r
import webbrowser

from stoffel.settings import *
from stoffel.core.ggl import Google
from stoffel.core.connect import *

os.chdir(os.path.dirname(os.path.abspath(__file__)))

app = Flask(__name__)

def read_json(file_path):
    if not os.path.isfile(CONNECTIONS_PATH):
        with open(CONNECTIONS_PATH, 'w+') as f:
            json.dump({}, f, indent=4)

    with open(CONNECTIONS_PATH, "rb+") as f:
        data = json.load(f)
    return data

def write_json(file_path, data):
    with open(CONNECTIONS_PATH, 'w+') as f:
        json.dump(data, f, indent=4)

@app.context_processor
def context_processor():
    data = read_json(CONNECTIONS_PATH)
    return dict(data=data)

@app.before_request
def load_data():
    data = read_json(CONNECTIONS_PATH)
    g.data = data

@app.route('/')
def home():
    if not os.path.exists(GOOGLE_TOKEN_PATH):
        Google()
    return render_template("home.html")

@app.route('/<source_destination>/<tp>')
def html_file(source_destination, tp):
    with open(f"templates/connections/{source_destination}/{tp}.html", "r") as file:
        data = file.read()
    return jsonify({"data" : data})

@app.route("/callback")
def callback():
    return redirect("/")

@app.route('/<name>', methods=["GET", "POST"])
def project(name):
    home = str(Path.home())
    if request.method == "POST":
        data = request.form.to_dict()
        project = dict()
        for key, value in data.items():
            connection_src_dest = key.split("_")[0]
            connection_id = key.split("_")[len(key.split("_"))-1]
            project.update({connection_id : {"source"  : dict(), "destination" : dict()}}) if connection_id not in project else None
            connection_key = "_".join(key.split("_")[1:-1])
            floats = ["w", "h", "x", "y"]
            if not value and connection_key in floats:
                value = 0
            project[connection_id][connection_src_dest][connection_key] = value if connection_key not in floats else float(value)
        g.data[name] = project
        write_json(CONNECTIONS_PATH, g.data)
        return redirect(f"/{name}")
    else:
        name = urllib.parse.unquote_plus(name)
        args = request.args.to_dict()
        errors = list()
        if name not in g.data:
            error = [f"Project {name} does not exist."]
        return render_template("project.html", project=name, errors=errors, args=args, home=home.replace("\\", "/").replace("C:", ""))

@app.route('/add-project/<name>', methods=['GET', 'POST'])
def add_project(name):
    name = urllib.parse.unquote_plus(name)
    if name in g.data.keys():
        name = name +  "(" + str(len([i for i in g.data.keys() if name in i])) + ")"
    g.data[name] = {}
    write_json(CONNECTIONS_PATH, g.data)
    return redirect(f"/{urllib.parse.quote_plus(name)}")

@app.route('/delete-project/<name>', methods=['GET', 'POST'])
def delete_project(name):
    name = urllib.parse.unquote_plus(name)
    g.data.pop(name, None)
    write_json(CONNECTIONS_PATH, g.data)
    return redirect(f"/")

@app.route('/run-project/<name>', methods=["GET"])
def run_project(name):
    name = urllib.parse.unquote_plus(name)
    project = Project(name)
    project.run()
    return redirect(f"/{name}")

@app.route('/open-file/<file_path>', methods=["GET"])
def open_file(file_path):
    name = urllib.parse.unquote_plus(file_path)
    os.startfile(name)
    return redirect(f"/")

@app.route('/google-url/<file_id>', methods=["GET"])
def google_url(file_id):
    ggl = Google()
    api_key = GOOGLE_PICKER_KEY
    headers = {
        "Authorization" : "Bearer " + ggl.creds.token, 
        "Accept" : "application/json" 
    }
    url = f"https://www.googleapis.com/drive/v3/files/{file_id}?key={GOOGLE_PICKER_KEY}&fields=webViewLink"
    response = r.get(url, headers=headers)
    return response.json()

@app.route("/file-tree", methods=["POST"])
def dirlist():
   r=['<ul class="jqueryFileTree" style="display: none;">']
   try:
       r=['<ul class="jqueryFileTree" style="display: none;">']
       d=urllib.parse.unquote(request.form.to_dict().get('dir','c:\\temp'))
       for f in os.listdir(d):
           ff=os.path.join(d,f)
           if os.path.isdir(ff):
               r.append('<li class="directory collapsed"><a href="#" rel="%s/">%s</a></li>' % (ff,f))
           else:
               e=os.path.splitext(f)[1][1:] # get .ext and remove dot
               r.append('<li class="file ext_%s"><a href="#" rel="%s">%s</a></li>' % (e,ff,f))
       r.append('</ul>')
   except Exception as e:
       r.append('Could not load directory: %s' % str(e))
   r.append('</ul>')
   return make_response(''.join(r))

if __name__=="__main__":
    url = "http://localhost:5500"
    webbrowser.open_new(url)
    app.run(port=5500, debug=False)


#Import packages. 
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
from apiclient.http import MediaIoBaseUpload, MediaIoBaseDownload
import pathlib
import pickle
import os
import io

from stoffel.core.utils.paths import path_leaf, gen_uuid
from stoffel.settings import *

class Slides:

    def __init__(self, presentation_id):
        self.slides = Google().slides()
        self.presentation_id = presentation_id
    
    def add_image(self, file_id, page_id, w, h, x, y):
        IMAGE_URL = f"https://drive.google.com/uc?export=view&id={file_id}"
        requests = []
        image_id = gen_uuid(1)
        requests.append({
            'createImage': {
                'objectId': image_id,
                'url': IMAGE_URL,
                'elementProperties': {
                    'pageObjectId': page_id,
                    'size': {
                        'height': {
                            "magnitude" : h * 72, 
                            "unit" : "PT"
                        },
                        'width': {
                            "magnitude" : w * 72,
                            "unit" : "PT"
                        }
                    },
                    "transform": {
                        "scaleX": 1,
                        "scaleY": 1,
                        "translateX": x * 72,
                        "translateY": y * 72,
                        "unit": "PT"
                    }
                }
            }
        })

        body = {
            'requests': requests
        }
        print(body)
        response = self.slides.presentations().batchUpdate(presentationId=self.presentation_id, body=body).execute()
        create_image_response = response.get('replies')[0].get('createImage')
        return create_image_response.get('objectId')

class Drive:

    def __init__(self, folder_id=None):
        self.drive = Google().drive()

    def create_folder(self, folder_name):
        folder_metadata = {
            "name": folder_name,
            "mimeType": "application/vnd.google-apps.folder"
        }
        folder = self.drive.files().create(body=folder_metadata, fields="id").execute()
        return folder

    def create_image(self, file_path, parent=None):
        print(f"Creating image for {file_path}")
        file_name = path_leaf(file_path)
        file_metadata = {'name': file_name}
        if parent:
            file_metadata["parents"] = [parent]
        content = open(file_path, 'rb')
        media_body = MediaIoBaseUpload(content,'application/octet-stream', resumable=True)
        f = self.drive.files().create(
            body=file_metadata,
            media_body=media_body,
            fields='webViewLink, id'
        ).execute()
        return f

    def find_file(self, file_name):
        result = []
        page_token = None
        while True:
            response = self.drive.files().list(q=f"name contains '{file_name}' and trashed=false",
                spaces='drive',
                fields='nextPageToken, files(id, name)',
                pageToken=page_token).execute()
            for file in response.get("files", []):
                result.append(file)
            page_token = response.get('nextPageToken', None)
            if not page_token:
                break
        return result

    def make_shareable(self, file_id):
        output = self.drive.permissions().create(
            fileId=file_id,
            body={
                "type" : "anyone", 
                "role" : "reader"
            },
            fields='id', 
        ).execute()
        return output

    def delete_file(self, file_id):
        return self.drive.files().delete(fileId=file_id).execute()

    def download_file(self, file_id, file_path, mime_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"):
        print(f"Downloading file {file_id} to {file_path}")
        request = self.drive.files().export_media(fileId=file_id, mimeType=mime_type)
        fh = io.FileIO(file_path, "wb")
        downloader = MediaIoBaseDownload(fh, request)
        done = False
        while done is False:
            status, done = downloader.next_chunk()
            print("Download %d%%." % int(status.progress() * 100))

class Google:

    def __init__(self):
        self.creds_path = GOOGLE_CREDS_PATH
        self.token_path = GOOGLE_TOKEN_PATH
        self.scopes = GOOGLE_SCOPES
        self.redirect= GOOGLE_REDIRECT
        self.creds = self.credentials()

    def credentials(self):
        creds = None
        if os.path.exists(self.token_path):
            with open(self.token_path, 'rb') as token:
                creds = pickle.load(token)
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    self.creds_path, scopes=self.scopes, redirect_uri=self.redirect)
                creds = flow.run_local_server(open_browser=True)
            with open(self.token_path, 'wb') as token:
                pickle.dump(creds, token)
        return creds

    def slides(self):
        service = build('slides', 'v1', credentials=self.creds, cache_discovery=False)  
        return service      

    def drive(self):
        service = build('drive', 'v3', credentials=self.creds, cache_discovery=False)  
        return service

if __name__=="__main__":
    #### ENV VARIABLES ####

    presentation_id = "1FJkzWBdb2AHL2-PKgu3BWFYtW9k1TK7OewTzvXX-BLI"
    slides = Slides(presentation_id)
    drive = Drive()
    folder = drive.create_folder("PantherTemp")
    print(drive.find_file("PantherTemp"))
    #directory = pathlib.Path(__file__).parent.parent.absolute()
    #f = drive.create_image(str(directory / "photos" / "test2_test_B3_D13.jpg"), parent=folder.get("id"))
    #print(drive.make_shareable(f.get("id")))
    #page_id = "g7d4b0c9081_0_22"
    #print(slides.add_image(f.get("id"), page_id))
    #print(drive.delete_file(f.get("id")))
    #sheet_id ="1Q1cm8MxDq0bzmok9PYzQc2ZjGSzNYRvjg5Yqggi_0f4"
    #drive.download_file(sheet_id)



from stoffel.settings import *
import os
import json

os.chdir(os.path.dirname(os.path.abspath(__file__)))

if not os.path.isfile(CONNECTIONS_PATH):
    with open(CONNECTIONS_PATH, 'w+') as f:
        json.dump({}, f, indent=4)


SOURCES = {
    "fields" : {
        "excel" : ["sheet", "rng", "excel_path"], 
        "sheets" : ["sheet", "rng", "sheet_id"]
    }
}
DESTINATIONS = {
    "fields" : {
        "slides" : ["w", "h", "x", "y", "presentation_id", "page_id"], 
        "powerpoint" : ["w", "h", "x", "y","presentation_path", "slide_index", "slide_id"]
    }
}


import ntpath
import uuid

def path_leaf(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)

def gen_uuid(lst_length):
  output = []
  for i in range(0, lst_length):
    output.append(str(uuid.uuid4()))

  if len(output) == 1:
    return output[0]
  else:
    return output


import sys
from setuptools import setup, Command

if sys.version_info[0] != 3:
    print('Stoffel requires Python 3 to run.')
    sys.exit(1)

setup(
    name='Stoffel',
    version="0.1.1",
    description="A tool to connect Excel/Sheets to Powerpoint/Slides.",
    author='Nathan Duncan',
    author_email='nduncan@fifthpartners.com',
    url='https://github.com/fifth-partners/stoffel',
    license='LGPLv3',
    python_requires="!=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    install_requires=[
        "xlwings==0.20.8", 
        "datetime==4.3",
        "google-api-python-client==1.12.5",
        "google-auth-httplib2==0.0.4",
        "google-auth-oauthlib==0.4.1",
        "PyDrive==1.3.1",
        "tabulate==0.8.7",
        "flask==1.1.2",
        "pillow==8.0.1",
    ],
    download_url = 'https://github.com/fifth-partners/stoffel/archive/v0.1.1.zip',
    include_package_data=True, 
    packages=['stoffel'],
    entry_points={"console_scripts": ["stoffel=stoffel.__main__:main"]},
    classifiers=[
        'Intended Audience :: Developers',
    ]
)


import os

CONNECTIONS_PATH="data.json"
GOOGLE_PICKER_KEY="AIzaSyDNRPFphQ7TxAqm7IgLp_C8wrFQCYCTjww"
GOOGLE_SCOPES=["https://www.googleapis.com/auth/drive", "https://www.googleapis.com/auth/presentations"]
GOOGLE_REDIRECT="http://localhost:5500/callback"
GOOGLE_CREDS_PATH="creds/credentials.json"
GOOGLE_TOKEN_PATH="creds/token.pickle"
GOOGLE_TOKEN_JSON="creds/token.json"
PROJECT_NAME="stoffel"




import PIL
import os
import pathlib
import xlwings as xw
import datetime as dt

from win32com.client import DispatchEx
import pythoncom

from stoffel.core.utils.vba import RANGE_TO_IMAGE
from stoffel.core.utils.paths import path_leaf

class Excel:
    def __init__(self, file_path):
        self.file_path = file_path

    def open(self, visible=False):
        print(f"Opening file: {self.file_path}")
        pythoncom.CoInitialize()
        self.app = xw.App(visible=visible, add_book=False)
        self.wb = self.app.books.open(self.file_path)

    def add_vba(self):
        print(f"Adding VBA")
        xlmodule = self.wb.api.VBProject.VBComponents.Add(1)
        xlmodule.CodeModule.AddFromString(RANGE_TO_IMAGE.strip())

    def run_vba(self, macro_name, *args):
        print(f"Running VBA")
        self.app.api.Run(macro_name, *args)

    def close(self):
        print("Closing file")
        self.wb.close()

    def get_image(self, sht, rng, file_path):
        self.open()
        self.add_vba()
        self.run_vba("RangetoImage", sht, rng, file_path)
        print(f"Saved image to: {file_path}")
        self.close()

    def find_images(self):
        self.open()
        for sheet in self.wb.sheets:
            print(sheet.name)
            myCell = self.wb.sheets[sheet.name].api.UsedRange.Find('x', LookAt=1)
            print(myCell.address)

    def delete(self):
        if os.path.exists(self.file_path):
            os.remove(self.file_path) 

class Powerpoint:
    
    def __init__(self, file_path):
        self.file_path = file_path
        self.file_name, self.file_extension = path_leaf(file_path).split(".")
        self.directory = pathlib.Path(__file__).parent.absolute()

    def open(self):
        print(self.file_path)
        pythoncom.CoInitialize()
        self.app = DispatchEx('PowerPoint.Application')
        self.pt = self.app.Presentations.Open(self.file_path, WithWindow=False)

    def close(self):
        self.pt.Close()

    def save(self):
        self.pt.Save()

    def add_image(self, picture_path, slide_id, w, h, x, y):
        self.open()
        slide = self.pt.Slides.FindBySlideID(slide_id)
        slide.Shapes.AddPicture(FileName=picture_path, LinkToFile=False, SaveWithDocument=True, Left=x*72, Top=y*72, Width=w*72, Height=h*72)
        self.save()
        self.close()

    def get_slide_id(self, slide_index):
        return self.pt.Slides(int(slide_index)).SlideID

if __name__=="__main__":
    
    directory = pathlib.Path(__file__).parent.parent.absolute()
    '''
    wb = Excel(str(directory / "test" / "1Q1cm8MxDq0bzmok9PYzQc2ZjGSzNYRvjg5Yqggi_0f4.xlsx"))
    wb.open(visible=False)
    wb.add_vba()
    wb.run_vba("RangetoImage", "Sheet2", "B3:E7", str(directory / "test.jpg"))
    '''
    pt = Powerpoint(str(directory / "test" / "Presentation1.pptx"))
    pt.open()
    print(pt.Slides)




    